module.exports = {
    metadata: {
        family: "outlet",
        plugin: "outlet",
        label: "Plugwise © Smile/Stealth Outlet",
        tangible: true,
        discoverable: true,
        state: [{
            id: "switch",
            label: "Switch",
            type: {
                id: "boolean"
            }
        }, {
            id: "consumption",
            label: "Consumption",
            type: {
                id: "decimal"
            }
        }],
        actorTypes: [],
        sensorTypes: [],
        services: [{
            id: "on",
            label: "On"
        }, {
            id: "off",
            label: "Off"
        }, {
            id: "toggle",
            label: "Toggle"
        }],
        configuration: []
    },
    create: function () {
        return new Outlet();
    },
    discovery: function (options) {
        var discovery = new OutletDiscovery();

        discovery.options = options;

        return discovery;
    }
};

var q = require('q');

function OutletDiscovery() {
    /**
     *
     * @param options
     */
    OutletDiscovery.prototype.start = function () {
        if (this.node.isSimulated()) {
        } else {
        }
    };

    /**
     *
     * @param options
     */
    OutletDiscovery.prototype.stop = function () {
    };
}

/**
 *
 */
function Outlet() {
    /**
     *
     */
    Outlet.prototype.start = function () {
        var deferred = q.defer();

        this.state = {switch: false, consumption: 0};

        if (this.isSimulated()) {
            this.simulationInterval = setInterval(function () {
                if (this.state.switch) {
                    this.state.consumption += new Date().getTime() % 4;
                }

                this.publishStateChange();
            }.bind(this), 10000);

            deferred.resolve();
        } else {
            deferred.resolve();
        }

        return deferred.promise;
    };

    /**
     *
     */
    Outlet.prototype.stop = function () {
        if (this.isSimulated()) {
            clearInterval(this.simulationInterval);
        }
    };

    /**
     *
     */
    Outlet.prototype.setState = function (state) {
        this.state = state;

        this.publishStateChange();
    };

    /**
     *
     */
    Outlet.prototype.getState = function () {
        return this.state;
    };

    /**
     *
     */
    Outlet.prototype.on = function () {
        this.state.switch = true;

        this.publishStateChange();
    };

    /**
     *
     */
    Outlet.prototype.off = function () {
        this.state.switch = false;

        this.publishStateChange();
    };

    /**
     *
     */
    Outlet.prototype.toggle = function () {
        this.state.switch = !this.state.switch;

        this.publishStateChange();
    };
}
